<?php

/**
 * @file
 * Provides infrequently used functions and hooks for feed_block.
 */

/**
 * Menu callback: display the menu block addition form.
 *
 * @see feed_block_add_block_form_submit()
 */
function feed_block_add_block_form($form, &$form_state) {
  module_load_include('inc', 'block', 'block.admin');
  $form = block_admin_configure($form, $form_state, 'feed_block', NULL);

  // Other modules should be able to use hook_form_block_add_block_form_alter()
  // to modify this form, so add a base form ID.
  $form_state['build_info']['base_form_id'] = 'block_add_block_form';
  $form = $form + _feed_block_additional_form_elements($form_state);
  $form['regions']['#collapsible'] = TRUE;
  $form['regions']['#collapsed'] = TRUE;
  // Prevent block_add_block_form_validate/submit() from being automatically
  // added because of the base form ID by providing these handlers manually.
  $form['#validate'] = [];
  $form['#submit'] = ['feed_block_add_block_form_submit'];

  return $form;
}

/**
 * Custom fields for this block type.
 *
 * @param mixed $form_state
 *    The Drupal form state.
 *
 * @return array
 *    Additional/modified Form API elements.
 */
function _feed_block_additional_form_elements($form_state) {
  $config = [];
  if (isset($form_state['values'])) {
    $delta = $form_state['values'];
    $config = variable_get("feed_block_{$delta}_config", []);
  }
  $form = [];
  $form['settings']['custom'] = [
    '#type' => 'fieldset',
    '#weight' => -19,
  ];
  $form['settings']['title']['#weight'] = -21;
  $form['settings']['custom']['intro_text'] = [
    '#type' => 'text_format',
    '#title' => t('Introductory text'),
    '#default_value' => isset($config['intro_text']['value']) ? $config['intro_text']['value'] : '',
    '#format' => isset($config['intro_text']['format']) ? $config['intro_text']['format'] : 'plain_text',
    '#description' => t('Optional text to display above the feed items.'),
    '#weight' => -20,
  ];
  $form['settings']['custom']['feed_url'] = [
    '#type' => 'textfield',
    '#title' => t('Feed URL'),
    '#default_value' => isset($config['feed_url']) ? $config['feed_url'] : '',
    '#description' => t('Enter the full URL to the RSS feed'),
    '#weight' => -19,
    '#required' => TRUE,
    '#element_validate' => ['feed_block_form_feed_validate'],
  ];
  $form['settings']['custom']['display_count'] = [
    '#type' => 'select',
    '#title' => t('Number of items to display'),
    '#options' => array_combine(range(1, 10), range(1, 10)),
    '#default_value' => isset($config['display_count']) ? $config['display_count'] : 5,
    '#weight' => -18,
  ];
  $form['settings']['custom']['header'] = [
    '#type' => 'checkbox',
    '#title' => t('Display block title as header'),
    '#default_value' => isset($config['header']) ? $config['header'] : 0,
    '#weight' => -17,
  ];
  $form['settings']['custom']['date'] = [
    '#type' => 'checkbox',
    '#title' => t('Display date'),
    '#default_value' => isset($config['date']) ? $config['date'] : 0,
    '#weight' => -16,
  ];
  // Get list of available formats.
  $formats = system_get_date_formats();
  $choices = [];
  foreach ($formats as $type => $list) {
    foreach ($list as $f => $format) {
      $choices[$f] = format_date(REQUEST_TIME, 'custom', $f);
    }
  }
  reset($formats);
  // Show date format select list.
  $form['settings']['custom']['date_format'] = [
    '#type' => 'select',
    '#title' => 'Date display format',
    '#attributes' => ['class' => ['date-format']],
    '#default_value' => isset($config['date_format']) ? $config['date_format'] : 'j F Y - g:ia',
    '#options' => $choices,
    '#weight' => -15,
    '#states' => [
      'invisible' => [
        ':input[name="date"]' => ['checked' => FALSE],
      ],
    ],
  ];

  $form['settings']['custom']['description'] = [
    '#type' => 'checkbox',
    '#title' => t('Display description'),
    '#default_value' => isset($config['description']) ? $config['description'] : 0,
    '#weight' => -13,
  ];
  $form['settings']['custom']['description_length'] = [
    '#type' => 'textfield',
    '#title' => t('Description length (in characters)'),
    '#default_value' => isset($config['description_length']) ? $config['description_length'] : '',
    '#description' => t('For no trimming, leave this field blank'),
    '#weight' => -12,
    '#states' => [
      'invisible' => [
        ':input[name="description"]' => ['checked' => FALSE],
      ],
    ],
  ];
  $form['settings']['custom']['strip_html'] = [
    '#type' => 'checkbox',
    '#title' => t('Remove HTML markup from description text.'),
    '#default_value' => isset($config['strip_html']) ? $config['strip_html'] : 0,
    '#weight' => -11,
    '#states' => [
      'invisible' => [
        ':input[name="description"]' => ['checked' => FALSE],
      ],
    ],
  ];
  $form['settings']['custom']['read_more'] = [
    '#tree' => TRUE,
    '#type' => 'link_field',
    '#field_parents' => [],
    '#field_name' => 'read_more',
    '#delta' => 0,
    '#language' => LANGUAGE_NONE,
    '#title' => t('Read more text'),
    '#default_value' => isset($config['read_more']) ? $config['read_more'] : '',
    '#description' => t('Optionally, a URL and display text to direct the user
      to the source site.'),
    '#weight' => -10,
  ];
  $form['settings']['custom']['cache'] = [
    '#type' => 'textfield',
    '#title' => t('Feed cache lifetime (in seconds)'),
    '#default_value' => isset($config['cache']) ? $config['cache'] : 300,
    '#description' => t('How long feed data should be kept before being considered stale. Generally, this should stay higher that the page cache lifetime.'),
    '#weight' => -9,
  ];
  return $form;
}

/**
 * Called by feed_block_block_configure().
 */
function _feed_block_block_configure($delta = '') {
  // Create a pseudo form state.
  $form_state = ['values' => $delta];
  return feed_block_configure_form([], $form_state);
}

/**
 * Returns the configuration form for a menu tree.
 *
 * @param mixed $form_state
 *   array An associated array of configuration options should be present in the
 *   'values' key. If none are given, default configuration is assumed.
 *
 * @return array
 *   array The form in Form API format.
 */
function feed_block_configure_form($form, &$form_state) {
  return $form + _feed_block_additional_form_elements($form_state);
}

/**
 * Save the new menu block.
 */
function feed_block_form_feed_validate($form, &$form_state) {
  $feed_url = $form_state['values']['feed_url'];
  $use_errors = libxml_use_internal_errors(TRUE);
  $xml = @simplexml_load_file($feed_url);
  if (FALSE === $xml) {
    form_set_error('feed_url', t('The URL "@url" does not appear to be a valid feed. The block has not been saved. Consider checking your feed using the !validator.', [
      '@url' => $feed_url,
      '!validator' => l(t('WC3 feed validator'), 'https://validator.w3.org/feed/'),
    ]));
  }
  libxml_clear_errors();
  libxml_use_internal_errors($use_errors);
}

/**
 * Save the new menu block.
 */
function feed_block_add_block_form_submit($form, &$form_state) {
  // Determine the delta of the new block.
  $block_ids = variable_get('feed_block_ids', []);
  $delta = empty($block_ids) ? 1 : max($block_ids) + 1;
  $form_state['values']['delta'] = $delta;

  // Save the new array of blocks IDs.
  $block_ids[] = $delta;
  variable_set('feed_block_ids', $block_ids);

  // Save the block configuration.
  feed_block_block_save($delta, $form_state['values']);

  // Run the new block submission (borrowed from block_add_block_form_submit).
  $query = db_insert('block')->fields([
    'visibility',
    'pages',
    'custom',
    'title',
    'module',
    'theme',
    'region',
    'status',
    'weight',
    'delta',
    'cache',
  ]);
  foreach (list_themes() as $key => $theme) {
    if ($theme->status) {
      $region = !empty($form_state['values']['regions'][$theme->name]) ? $form_state['values']['regions'][$theme->name] : BLOCK_REGION_NONE;
      $query->values([
        'visibility' => (int) $form_state['values']['visibility'],
        'pages' => trim($form_state['values']['pages']),
        'custom' => (int) $form_state['values']['custom'],
        'title' => $form_state['values']['title'],
        'module' => $form_state['values']['module'],
        'theme' => $theme->name,
        'region' => ($region == BLOCK_REGION_NONE ? '' : $region),
        'status' => 0,
        'status' => (int) ($region != BLOCK_REGION_NONE),
        'weight' => 0,
        'delta' => $delta,
        'cache' => DRUPAL_NO_CACHE,
      ]);
    }
  }
  $query->execute();

  $query = db_insert('block_role')->fields(['rid', 'module', 'delta']);
  foreach (array_filter($form_state['values']['roles']) as $rid) {
    $query->values([
      'rid' => $rid,
      'module' => $form_state['values']['module'],
      'delta' => $delta,
    ]);
  }
  $query->execute();

  drupal_set_message(t('An RSS Feed block has been created.'));
  $form_state['redirect'] = 'admin/structure/block';
}

/**
 * Alters the block admin form to add delete links next to RSS feed blocks.
 */
function _feed_block_form_block_admin_display_form_alter(&$form, $form_state) {
  $ids = [];
  module_load_include('inc', 'block', 'block.admin');
  global $theme_key;
  $blocks = block_admin_display_prepare_blocks($theme_key);
  foreach ($blocks as $block) {
    if ($block['module'] == 'feed_block') {
      $ids[] = $block['delta'];
    }
  }
  foreach ($ids as $delta) {
    $form['blocks']['feed_block_' . $delta]['delete'] = [
      '#type' => 'link',
      '#title' => t('delete'),
      '#href' => 'admin/structure/block/delete-feed-block/' . $delta,
    ];
  }
}

/**
 * Menu callback: confirm deletion of menu blocks.
 */
function feed_block_delete_form($form, &$form_state, $delta = 0) {
  $block = block_load('feed_block', $delta);
  $title = $block->title;
  $form['block_title'] = ['#type' => 'hidden', '#value' => $title];
  $form['delta'] = ['#type' => 'hidden', '#value' => $delta];

  return confirm_form($form, t('Are you sure you want to delete the "%name" block?', ['%name' => $title]), 'admin/structure/block', NULL, t('Delete'), t('Cancel'));
}

/**
 * Deletion of menu blocks.
 */
function feed_block_delete_form_submit($form, &$form_state) {
  // Remove the menu block configuration variables.
  $delta = $form_state['values']['delta'];
  feed_block_delete($delta);
  drupal_set_message(t('The block "%name" has been removed.', ['%name' => $form_state['values']['block_title']]));
  $form_state['redirect'] = 'admin/structure/block';
}

/**
 * Business logic for feed_block_block_info().
 */
function _feed_block_block_info() {
  $blocks = [];
  $deltas = variable_get('feed_block_ids', []);
  $deltas = array_unique($deltas);
  foreach ($deltas as $delta) {
    $block = block_load('feed_block', $delta);
    $blocks[$delta]['info'] = $block->title . ' ' . t('(RSS Feed)');
    // Blocks can't be cached because its content is dynamic.
    $blocks[$delta]['cache'] = DRUPAL_NO_CACHE;
  }
  return $blocks;
}

/**
 * Implements hook_block_save().
 */
function _feed_block_block_save($delta = '', $edit = []) {
  if (!empty($delta)) {
    $config['feed_url'] = $edit['feed_url'];
    $config['intro_text'] = $edit['intro_text'];
    $config['read_more'] = $edit['read_more'];
    $config['header'] = $edit['header'];
    $config['date'] = $edit['date'];
    $config['date_format'] = $edit['date_format'];
    $config['description'] = $edit['description'];
    $config['description_length'] = $edit['description_length'];
    $config['strip_html'] = $edit['strip_html'];
    $config['display_count'] = $edit['display_count'];
    $config['cache'] = $edit['cache'];
    variable_set("feed_block_{$delta}_config", $config);
  }
}
